package com.example.witsystem_device_plugin.SDK.event;


import android.bluetooth.BluetoothGatt;
import android.os.Handler;

import com.alibaba.fastjson.JSON;
import com.witsystem.top.witsystem_android_ble_sdk.WitsystemSDK;
import com.witsystem.top.witsystem_android_ble_sdk.WitsystmBleSDK;
import com.witsystem.top.witsystem_android_ble_sdk.ble.call.ConnectionStateChangeCall;
import com.witsystem.top.witsystem_android_ble_sdk.device.BleDevice;
import com.witsystem.top.witsystem_android_ble_sdk.error.InitErrorCode;
import com.witsystem.top.witsystem_android_ble_sdk.error.NotInitialized;

import java.util.Map;
import java.util.TreeMap;

import io.flutter.plugin.common.EventChannel;

/**
 * 处理蓝牙连接状态
 */
public class BleConnectStateHandler implements EventChannel.StreamHandler, ConnectionStateChangeCall {

    private EventChannel.EventSink events;

    private final Handler handler = new Handler();

    private BleDevice bleDevice;

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.events = events;
       // Log.i("蓝牙连接", "onListen: 启动蓝牙监听");
        WitsystemSDK witsystemSDK = WitsystmBleSDK.getInstance(null, null, null, null, null);
        if (witsystemSDK == null) {
            handler.post(() -> events.error(String.valueOf(InitErrorCode.NOT_INITIALIZED.getCode()), InitErrorCode.NOT_INITIALIZED.getError(), JSON.toJSONString(new NotInitialized())));
            return;
        }
        bleDevice = witsystemSDK.getBleDevice();
        bleDevice.bleDeviceConnectState(this);
    }

    @Override
    public void onCancel(Object arguments) {
        //Log.i("蓝牙连接", "onCancel: 关闭蓝牙监听");
        events.endOfStream();
        if(bleDevice!=null)bleDevice.cancelBleDeviceConnectState(this);
    }


    @Override
    public void onConnectionStateChange(BluetoothGatt bluetoothGatt, int i, int i1) {
        Map<String, Object> map = new TreeMap<>();
        map.put("deviceId", bluetoothGatt.getDevice().getName());
        map.put("mac", bluetoothGatt.getDevice().getAddress());
        map.put("oldVal", i);
        map.put("newVal", i1);
        handler.post(() -> events.success(JSON.toJSONString(map)));
    }
}
