import Flutter
import UIKit

import WitsystemIosSDK

public class SwiftWitsystemDevicePlugin: NSObject, FlutterPlugin {

    static var channel: FlutterMethodChannel? = nil;

    ///扫描添加设备
    static var addDeviceEventChannel: FlutterEventChannel? = nil;
    static var addDeviceHandler: AddDeviceHandler? = nil;

    //static var flutterViewController: FlutterViewController? = nil

    public static func register(with registrar: FlutterPluginRegistrar) {
        channel = FlutterMethodChannel(name: ChannelName.witsystemSDK, binaryMessenger: registrar.messenger())
        let instance = SwiftWitsystemDevicePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel!)

        //let flutterEngine:FlutterEngine = FlutterEngine(name: "my flutter engine")
        //flutterViewController = FlutterViewController(engine: flutterEngine, nibName: nil, bundle: nil)
        //addDeviceEventChannel = FlutterEventChannel(name: "com.pages.flutter", binaryMessenger: flutterViewController as! FlutterBinaryMessenger)

        addDeviceEventChannel = FlutterEventChannel(name: ChannelName.multipleAddDevice, binaryMessenger: registrar.messenger())
        addDeviceHandler = AddDeviceHandler()
        addDeviceEventChannel!.setStreamHandler(addDeviceHandler as (FlutterStreamHandler & NSObjectProtocol)?)

        let bleConnectStateEventChannel = FlutterEventChannel(name: ChannelName.bleConnectState, binaryMessenger: registrar.messenger())
        let bleConnectStateHandler = BleConnectStateHandler()
        bleConnectStateEventChannel.setStreamHandler(bleConnectStateHandler as (FlutterStreamHandler & NSObjectProtocol)?)


        let deviceChangeEventChannel = FlutterEventChannel(name: ChannelName.deviceChange, binaryMessenger: registrar.messenger())
        let deviceChangeHandler = DeviceChangeHandler()
        deviceChangeEventChannel.setStreamHandler(deviceChangeHandler as (FlutterStreamHandler & NSObjectProtocol)?)

        //registrar
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if (call.method != "initSDK" && !WitsystemIosSDK.witsystemIosSDK.isInit) {
            result(FlutterError(code: String(InitErrorCode.NOT_INITIALIZED.rawValue), message: InitErrorCode.NOT_INITIALIZED.toError(), details: nil))
            return;
        }
        print(">>>>>>>", call.method);
        print(">>>>>>>", call.arguments ?? "");
        let prm = call.arguments as? Dictionary<String, Any>
        switch call.method {
        case "initSDK": //初始化
            initSDK(appId: prm!["appId"]! as! String, token: prm!["token"]! as! String, appSecret: prm!["appSecret"]! as! String, rsaPrivateKey: prm!["rsaPrivateKey"]! as! String, result: result)
            break;
        case "uninitDevices"://取消初始化
            WitsystemIosSDK.witsystemIosSDK.uninitialized()
            result("SUCCESS");
            break;

        case "bleUnlock"://开启设备
            bleUnlock(deviceId: prm!["deviceId"] as? String, result: result);
            break;

        case "longUnlock"://远程开启设备
            longUnlock(deviceId: prm!["deviceId"] as? String, result: result)
            break;

        case "connectAndOpen"://设备连接认证和开启设备
            connectAndOpen(deviceId: prm!["deviceId"]! as! String, isOpen: prm!["isOpen"] == nil ? false : prm!["isOpen"]! as! Bool, result: result);
            break;

        case "scanAddBleDevice"://扫描添加蓝牙设备
            startAddScan()
            result("SUCCESS");
            break;

        case "stopAddBleDevice"://取消扫描添加蓝牙设备
            try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().stopScanAddBleDevice();
            result("SUCCESS");
            break;


        case "addBleDevice"://添加设备
            addDevice(deviceId: prm!["deviceId"]! as! String, deviceName: prm!["deviceName"]! as! String, result: result)
            break;

        case "closeAddDevice"://关闭添加设备
            try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().closeAddDevice(deviceId: prm!["deviceId"]! as! String)
            break;

        case "addPassword"://添加密码
            addPassword(deviceId: prm!["deviceId"]! as! String, passwordType: prm!["passwordType"]! as! String, password: prm!["password"]! as! String, passwordName: prm!["passwordName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;

        case "deletePassword"://删除密码
            deletePassword(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "addCard"://添加卡
            addCard(deviceId: prm!["deviceId"]! as! String, cardType: prm!["cardType"]! as! String, cardName: prm!["cardName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;

        case "cancelAddCard"://取消添加卡
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().cancelAddCard(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS");
            break;

        case "deleteCard"://删除卡
            deleteCard(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeCard"://冻结卡
            freezeCard(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "closeOrOpenCard"://关闭或者开启卡功能，
            closeOrOpenCard(deviceId: prm!["deviceId"]! as! String, isClose: prm!["isClose"]! as! Bool, result: result);
            break;

        case "addMark"://添加指纹
            addMark(deviceId: prm!["deviceId"]! as! String, markType: prm!["markType"]! as! String, markName: prm!["markName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;

        case "cancelAddMark"://取消添加指纹
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().cancelAddMark(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS");
            break;

        case "deleteMark"://删除指纹
            deleteMark(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeMark"://冻结指纹
            freezeMark(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "addFace"://添加人脸
            addFace(deviceId: prm!["deviceId"]! as! String, faceType: prm!["faceType"]! as! String, faceName: prm!["faceName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;


        case "cancelAddFace"://取消添加人脸
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().cancelAddFace(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS")
            break;

        case "deleteFace"://删除人脸
            deleteFace(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeFace"://冻结人脸
            freezeFace(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "updateVolume"://修改设备音量
            updateVolume(deviceId: prm!["deviceId"]! as! String, volume: prm!["volume"]! as! Int, result: result)
            break;

        case "setModel"://设置模式
            setModel(deviceId: prm!["deviceId"]! as! String, deviceModel: prm!["model"]! as! String, result: result)
            break;

        case "updateDeviceState"://设置模式
            updateDeviceState(deviceId: prm!["deviceId"]! as! String, result: result)
            break;

        case "closeBleDevice"://关闭蓝牙设备操作
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().closeBleDevice(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS")
            break;

        case "checkUpVersion"://检查是否有新版固件
            checkUpVersion(deviceId: prm!["deviceId"]! as! String, result: result)
            break;

        case "startBleOta"://蓝牙ota
            startBleOta(deviceId: prm!["deviceId"]! as! String, result: result)
            break;

        case "stopBleOta"://停止蓝牙ota
            stopBleOta(deviceId: prm!["deviceId"]! as! String, result: result)
            break;


        case "device"://获得指定设备
            let device = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(prm!["deviceId"]! as! String)
            result(device == nil ? nil : String(data: try! JSONEncoder().encode(device), encoding: .utf8))
            break;

        case "deviceList"://获得设备列表
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().deviceList()), encoding: .utf8))
            break;

        case "isDeviceFunction"://是否支持指定功能
            //print("是否支持？？？？？？？？？？",try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().isDeviceFunction(deviceId: prm!["deviceId"]! as! String, type: DeviceFunctionType(rawValue: prm!["type"]! as! String)!))
            result(try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().isDeviceFunction(deviceId: prm!["deviceId"]! as! String, type: DeviceFunctionType(rawValue: prm!["type"]! as! String)!))
            break;

        case "deviceUserList"://设备用户列表
            deviceUserList(deviceId: prm!["deviceId"]! as! String, result: result);
            break;

        case "freezeOrUnfreeze"://冻结用户
            freezeOrUnfreeze(deviceId: prm!["deviceId"]! as! String, freezeUserUuid: prm!["freezeUserUuid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result);
            break;

        case "manageDeleteDeviceUser"://管理员或者高等级删除低等级权限的用户的权限
            manageDeleteDeviceUser(deviceId: prm!["deviceId"]! as! String, deleteUserUuid: prm!["deleteUserUuid"]! as! String, result: result);
            break;

        case "translationDevice"://平移设备
            translationDevice(userId: prm!["userId"]! as! String, deviceId: prm!["deviceId"]! as! String, result: result);
            break;

        case "deleteDevices"://删除设备
            deleteDevices(deviceId: prm!["deviceId"]! as! String, result: result);
            break;

        case "lockRecord"://获得开锁日志
            lockRecord(deviceId: prm!["deviceId"]! as! String, page: prm!["page"]! as! Int, result: result);
            break;

        case "refurbishDevice"://刷新指定设备
            result(try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().refurbishDevice(deviceId: prm!["deviceId"]! as! String))
            break;

        case "refurbishAllDevice"://刷新全部设备
            result(try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().refurbishAllDevice())
            break;

        case "getDeviceStateInfo"://获得的设备状态
            let stateCode = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().getDeviceStateInfo(deviceId: prm!["deviceId"]! as! String)
            result(stateCode == nil ? nil : String(data: try! JSONEncoder().encode(stateCode), encoding: .utf8))
            break;

        case "getDeviceSoftwareFunction"://获得的设备软件支持信息
            let deviceSoftwareInfo = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().getDeviceSoftwareFunction(deviceId: prm!["deviceId"]! as! String)
            result(deviceSoftwareInfo == nil ? nil : String(data: try! JSONEncoder().encode(deviceSoftwareInfo), encoding: .utf8))
            break;

        case "updateDeviceName"://修改设备名称
            updateDeviceName(deviceId: prm!["deviceId"]! as! String, deviceName: prm!["deviceName"]! as! String, result: result);
            break;

        case "sendDevice"://发送设备
            sendDevice(userId: prm!["userId"]! as! String,
                    deviceId: prm!["deviceId"]! as! String,
                    startDate: prm!["startDate"]! as! String,
                    endDate: prm!["endDate"]! as! String,
                    authType: prm!["authType"]! as! String,
                    repeatType: prm!["repeatType"]! as! String,
                    result: result)
            break;


        case "bindRelay"://绑定网关
            bindRelay(superDeviceId: prm!["superDeviceId"]! as! String, deviceId: prm!["deviceId"]! as! String, result: result);
            break;

        case "unbindRelay"://解绑网关
            unbindRelay(deviceId: prm!["deviceId"]! as! String, result: result);
            break;

        case "startVibrate"://开始振动
            //建立的SystemSoundID对象,标准长震动
            // let soundID = SystemSoundID(kSystemSoundID_Vibrate)
            Vibrate.startVibrate(pattern: nil)
            result("SUCCESS")
            break;

        case "stopVibrate"://停止振动
            Vibrate.stopVibrate()
            result("SUCCESS")
            break;

        default:
            result(FlutterMethodNotImplemented)
            break;
        }


    }

    ///初始化SDK
    private func initSDK(appId: String, token: String, appSecret: String, rsaPrivateKey: String, result: @escaping FlutterResult) {
        WitsystemIosSDK.witsystemIosSDK.initSdk(appId: appId, token: token, appSecret: appSecret, rsaPrivateKey: rsaPrivateKey) { WitsystemIosSDK in
            print("swift初始化完成", result)
            result(true)
        } fail: { ErrorCode in
            print("swift初始化异常", result)
            result(FlutterError(code: String(ErrorCode.code), message: ErrorCode.error, details: nil));
        }
    }

    ///开启设备
    private func bleUnlock(deviceId: String?, result: @escaping FlutterResult) {
        class Info: Codable {
            var type: String = "UNLOCK"
            var devices: [Devices?]?

            init(type: String, devices: [Devices?]?) {
                self.type = type;
                self.devices = devices;
            }
        }

        func success(devices: Devices?, code: ErrorCode) {
            if (code.code == DeviceErrorCode.SUCCESS.code) {
                let data = try! JSONEncoder().encode(Info(type: "UNLOCK", devices: [devices]))
                result(String(data: data, encoding: .utf8))
            } else {
                result(FlutterError(code: String(code.code), message: code.error, details: nil))
            }
        }

        if (deviceId != nil) {
            try! WitsystemIosSDK.witsystemIosSDK.getUnlock().unlock(deviceId!, call: success)
        } else {
            try! WitsystemIosSDK.witsystemIosSDK.getUnlock().unlock(call: success, multipleDevices: { peripherals in
                let deviceManage = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage();
                var devices: [Devices?] = [];
                for index in 0..<peripherals.count {
                    devices.append(deviceManage.device(peripherals[index].name!))
                }
                let data = try! JSONEncoder().encode(Info(type: "UNLOCK", devices: devices))
                result(String(data: data, encoding: .utf8))
            })
        }

    }


    ///远程开门
    private func longUnlock(deviceId: String?, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getUnlock().longUnlock(deviceId: deviceId!)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }


    ///连接认证和开启设备
    private func connectAndOpen(deviceId: String, isOpen: Bool? = false, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().connectAndOpen(deviceId: deviceId, isOpen: isOpen ?? false) { devices, code in
            if (code.code == DeviceErrorCode.SUCCESS.code) {
                result("SUCCESS")
            } else {
                result(FlutterError(code: String(code.code), message: code.error, details: nil))
            }
        }
    }

    ///扫描添加设备
    private func startAddScan() {
        try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().scanAddBleDevice(scanDevice: { deviceId, dictionary, number in
            var mac = deviceId.suffix(12)
            mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 10))
            mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 8))
            mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 6))
            mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 4))
            mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 2))
            SwiftWitsystemDevicePlugin.addDeviceHandler?.onScanResult(
                    map: ["address": String(mac), "deviceId": deviceId, "rssi": number.debugDescription, "bytes": dictionary.description]
            )

        }, fail: { code in
            SwiftWitsystemDevicePlugin.addDeviceHandler?.onError(code: String(code.code), message: code.error)
        })
    }

    ///添加设备
    private func addDevice(deviceId: String, deviceName: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().addBleDevice(deviceId: deviceId, deviceName: deviceName, success: { devices in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        })
    }

    ///添加密码
    private func addPassword(deviceId: String, passwordType: String, password: String, passwordName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addPassword(deviceId: deviceId, passwordType: PasswordType(rawValue: passwordType) ?? PasswordType.ORDINARY_PASSWORD, password: password, passwordName: passwordName, userId: userId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///删除密码
    private func deletePassword(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deletePassword(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///添加卡
    private func addCard(deviceId: String, cardType: String, cardName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addCard(deviceId: deviceId, cardType: CardType(rawValue: cardType) ?? CardType.USER_CARD, cardName: cardName, userId: userId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, sendSuccess: {
            SwiftWitsystemDevicePlugin.channel?.invokeMethod("sendSuccess", arguments: nil)
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///删除卡
    private func deleteCard(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteCard(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结卡
    private func freezeCard(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeCard(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///关闭刷卡功能
    private func closeOrOpenCard(deviceId: String, isClose: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().closeOrOpenCard(deviceId: deviceId, isClose: isClose, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///添加指纹
    private func addMark(deviceId: String, markType: String, markName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addMark(deviceId: deviceId, markType: MarkType(rawValue: "markType") ?? MarkType.USER_MARK, markName: markName, userId: userId,
                success: { message in
                    result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
                },
                sendSuccess: {
                    SwiftWitsystemDevicePlugin.channel?.invokeMethod("sendSuccess", arguments: nil)
                }, progress: { sum, value in
            SwiftWitsystemDevicePlugin.channel?.invokeMethod("progress", arguments: String(data: try! JSONEncoder().encode(["sum": sum, "value": value]), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///删除指纹
    private func deleteMark(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteMark(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结指纹
    private func freezeMark(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeMark(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///添加人脸
    private func addFace(deviceId: String, faceType: String, faceName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addFace(deviceId: deviceId, faceType: FaceType(rawValue: faceType) ?? FaceType.USER_FACE, faceName: faceName, userId: userId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, sendSuccess: {

        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///删除人脸
    private func deleteFace(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteFace(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结人脸
    private func freezeFace(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeFace(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///修改设备音量
    private func updateVolume(deviceId: String, volume: Int, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateVolume(deviceId: deviceId, volume: volume, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///设置模式
    private func setModel(deviceId: String, deviceModel: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateModel(deviceId: deviceId, deviceModel: DeviceModel(rawValue: deviceModel) ?? DeviceModel.NORMAL_MODE, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///更新设备状态
    private func updateDeviceState(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateDeviceState(deviceId: deviceId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///检查是否有升级版本
    private func checkUpVersion(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getOta().checkUpVersion(deviceId: deviceId, success: { otaEntityResult in
            result(["newVersion": otaEntityResult.newVersion, "isNewVersion": otaEntityResult.isNewVersion, "newDeviceModel": otaEntityResult.newDeviceModel])
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        })

    }

    ///开始蓝牙升级
    private func startBleOta(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getOta().startBleOta(deviceId: deviceId,
                success: { version in
                    result(version)
                }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        }, progress: { value in
            SwiftWitsystemDevicePlugin.channel?.invokeMethod("progress", arguments: value)
        })
    }

    ///停止蓝牙ota
    private func stopBleOta(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getOta().stopBleOta(deviceId: deviceId)
        result("SUCCESS");
    }

    ///获得设备用户
    private func deviceUserList(deviceId: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().deviceUserList(deviceId: deviceId)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

    ///冻结用户
    private func freezeOrUnfreeze(deviceId: String, freezeUserUuid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().freezeOrUnfreeze(deviceId: deviceId, freezeUserUuid: freezeUserUuid, isFreeze: isFreeze)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

    ///管理员或者高等级删除低等级权限的用户的权限
    private func manageDeleteDeviceUser(deviceId: String, deleteUserUuid: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().manageDeleteDeviceUser(deviceId: deviceId, deleteUserUuid: deleteUserUuid)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

    ///平移设备
    private func translationDevice(userId: String, deviceId: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().translationDevice(userId: userId, deviceId: deviceId)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

    ///删除指定设备
    private func deleteDevices(deviceId: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().deleteDevices(deviceId: deviceId)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

    ///获得开门记录
    private func lockRecord(deviceId: String, page: Int, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().lockRecord(deviceId: deviceId, page: page)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }


    ///修改设备名称
    private func updateDeviceName(deviceId: String, deviceName: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().updateDeviceName(deviceId: deviceId, deviceName: deviceName)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }


    /**
     * 发送设备
     *
     * @param deviceId   设备ID
     * @param userId     接收者账号
     * @param startDate  开始时间
     * @param endDate    结束时间
     * @param authType   权限类型
     * @param repeatType 重复类型
     */
    private func sendDevice(userId: String, deviceId: String, startDate: String, endDate: String, authType: String, repeatType: String, result: @escaping FlutterResult) {
        var map = ["userId": userId, "deviceId": deviceId, "authType": authType];
        map["startDate"] = startDate;
        map["endDate"] = endDate;
        map["repeatType"] = repeatType;

        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().sendDevice(userId: userId, deviceId: deviceId, startDate: startDate, endDate: endDate, authType: AuthType(rawValue: authType)!, repeatType: RepeatType(rawValue: repeatType)!);
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }

    }


///绑定网关
    private func bindRelay(superDeviceId: String, deviceId: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().bindRelay(superDeviceId: superDeviceId, deviceId: deviceId)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }

///解绑网关
    private func unbindRelay(deviceId: String, result: @escaping FlutterResult) {
        let netResponse = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage().unbindRelay(deviceId: deviceId)
        if (netResponse._err == NetErrorCode.SUCCESS_REQUEST.rawValue) {
            result(String(data: try! JSONEncoder().encode(netResponse), encoding: .utf8))
        } else {
            result(FlutterError(code: String(netResponse._err), message: netResponse._msg, details: nil))
        }
    }


}
