import 'dart:async';

import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/MethodName.dart';
import 'dart:convert' as convert;
import '../WitsystemSDK.dart';

///设备固件升级

class Ota extends WisystemSDk {
  StreamSubscription? _otaListen;

  final Stream<Map<String, dynamic>> _bleOtaEvent =
      const EventChannel(ChannelName.bleOta)
          .receiveBroadcastStream()
          .map((event) => convert.json.decode(event.toString()));

  factory Ota() => _getInstance();
  static Ota? _instance;

  Ota._internal();

  static Ota _getInstance() {
    _instance ??= Ota._internal();
    return _instance!;
  }

  ///检测是否有新版本
  Future<Map> checkUpVersion(String deviceId) async {
    return await methodChannel
        .invokeMethod("checkUpVersion", {"deviceId": deviceId});
  }

  ///启动蓝牙ota
  Future startBleOta({
    required String deviceId,
    required Function(double value) progress,
  }) async {
    methodChannel.setMethodCallHandler((call) {
      if (call.method == "progress") {
        progress(call.arguments);
      }
      return Future.value("SUCCESS");
    });
    try {
      return methodChannel.invokeMethod("startBleOta", {"deviceId": deviceId});
    } catch (e) {
      methodChannel.setMethodCallHandler(null);
      rethrow;
    }
  }

  ///停止蓝牙ota
  stopBleOta(String deviceId) async {
    return await methodChannel
        .invokeMethod("stopBleOta", {"deviceId": deviceId});
  }
}
